diff -u --recursive empeg.v2.00beta13.hijack/arch/arm/special/empeg_audio3.c empeg.v2.00beta13.audiooverlay/arch/arm/special/empeg_audio3.c
--- empeg.v2.00beta13.hijack/arch/arm/special/empeg_audio3.c	Thu Nov 21 12:51:42 2002
+++ empeg.v2.00beta13.audiooverlay/arch/arm/special/empeg_audio3.c	Fri Nov 29 23:09:52 2002
@@ -99,8 +99,10 @@
 /* debug */
 #define AUDIO_DEBUG			0
 #define AUDIO_DEBUG_VERBOSE		0
+#define AUDIO_OVERLAY_DEBUG			0
 #define AUDIO_DEBUG_STATS		1 //AUDIO_DEBUG | AUDIO_DEBUG_VERBOSE
 
+
 /* Names */
 #define AUDIO_NAME			"audio-empeg"
 #define AUDIO_NAME_VERBOSE		"empeg dsp audio"
@@ -112,6 +114,22 @@
 #define AUDIO_NOOF_BUFFERS		8	/* Number of audio buffers */
 #define AUDIO_BUFFER_SIZE		4608	/* User buffer chunk size */
 
+/* Audio overlay specific variables */
+#define AUDIO_OVERLAY_BUFFERS				(16)
+#define MAX_FREE_OVERLAY_BUFFERS			(AUDIO_OVERLAY_BUFFERS - 2)
+#define AUDIO_OVERLAY_BG_VOLUME_FADED		(0x00004000)
+#define AUDIO_OVERLAY_BG_VOLUME_MAX			(0x00010000)
+#define AUDIO_OVERLAY_BG_VOLUME_FADESTEP	(0x00000AAA)
+
+extern int hijack_overlay_bg_min;
+extern int hijack_overlay_bg_max;
+extern int hijack_overlay_bg_fadestep;
+
+
+int audio_overlay_bg_volume = AUDIO_OVERLAY_BG_VOLUME_MAX;
+signed short audio_zero_sample[ AUDIO_BUFFER_SIZE/2 ] = { 0 };
+extern int sam;
+
 /* Number of audio buffers that can be in use at any one time. This is
    two less since the inactive two are actually still being used by
    DMA while they look like being free. */
@@ -555,6 +573,12 @@
 	int  count;
 } audio_buf;
 
+struct
+{
+	audio_buf *buffers;
+	int used,free,head,tail,initialized;
+} audio_overlay;
+
 typedef struct
 {
 	/* Buffers */
@@ -575,6 +599,12 @@
 
 	/* Are we sending "good" data? */
 	int good_data;
+
+    /* overlay: has SAM been set yet */
+    int sam_set;
+
+    /* overlay: prev soft audio mute status */
+    int prev_sam;
 } audio_dev;
 
 /* cosine tables for beep parameters */
@@ -646,7 +676,18 @@
 
 	/* Blank everything to start with */
 	memset(dev, 0, sizeof(audio_dev));
-	
+
+    /*
+	// initialize audio overlay buffers
+	if ((audio_overlay.buffers=kmalloc(sizeof(audio_buf)*AUDIO_OVERLAY_BUFFERS,GFP_KERNEL))==NULL)
+		printk(AUDIO_NAME ": can't get memory for audio overlay buffers");
+	for(i = 0; i < AUDIO_OVERLAY_BUFFERS; i++)
+		audio_overlay.buffers[i].count = 0;
+	audio_overlay.head = audio_overlay.tail = audio_overlay.used = 0;
+	audio_overlay.free = MAX_FREE_OVERLAY_BUFFERS;
+	*/
+    audio_overlay.initialized = 0;
+
 	/* Allocate buffers */
 	if ((dev->buffers = kmalloc(sizeof(audio_buf) * AUDIO_NOOF_BUFFERS,
 				    GFP_KERNEL)) == NULL) {
@@ -824,6 +865,63 @@
 		return 0;
 	}
 
+	if( (file->f_flags & O_SYNC) )
+	{
+      if ( !audio_overlay.initialized )
+      {
+        int i;
+        unsigned long flags;
+#if AUDIO_OVERLAY_DEBUG
+        printk("audio overlay initializing\n");
+#endif
+
+        save_flags_cli(flags);
+        if ((audio_overlay.buffers=kmalloc(sizeof(audio_buf)*AUDIO_OVERLAY_BUFFERS,GFP_KERNEL))==NULL)
+            printk(AUDIO_NAME ": can't get memory for audio overlay buffers");
+        for(i = 0; i < AUDIO_OVERLAY_BUFFERS; i++)
+            audio_overlay.buffers[i].count = 0;
+        audio_overlay.head = audio_overlay.tail = audio_overlay.used = 0;
+        audio_overlay.free = MAX_FREE_OVERLAY_BUFFERS;
+        restore_flags(flags);
+        audio_overlay.initialized = 1;
+#if AUDIO_OVERLAY_DEBUG
+        printk("audio overlay initialized\n");
+#endif
+      }
+
+		if (audio_overlay.free==0) {
+		    struct wait_queue wait = { current, NULL };
+	
+		    add_wait_queue(&dev->waitq, &wait);
+		    current->state = TASK_INTERRUPTIBLE;
+		    while (audio_overlay.free == 0) {
+			schedule();
+		    }
+		    current->state = TASK_RUNNING;
+		    remove_wait_queue(&dev->waitq, &wait);
+
+		}
+		// fill data to overlay buffers instead
+		while( count > 0 && audio_overlay.free > 0 )
+		{
+			unsigned long flags;
+			save_flags_cli(flags);
+			audio_overlay.free--;
+			restore_flags(flags);
+
+			copy_from_user( audio_overlay.buffers[ audio_overlay.head++ ].data, buffer, AUDIO_BUFFER_SIZE );
+			if( audio_overlay.head == AUDIO_OVERLAY_BUFFERS )
+				audio_overlay.head = 0;
+			total += AUDIO_BUFFER_SIZE;
+			count -= AUDIO_BUFFER_SIZE;
+			
+			save_flags_cli(flags);
+			audio_overlay.used++;
+			restore_flags(flags);
+		}
+		return total;
+	}
+
 	/* Any space left? (No need to disable IRQs: we're just checking for a
 	   full buffer condition) */
 	/* This version doesn't have races, see p209 of Linux Device Drivers */
@@ -1100,6 +1198,253 @@
 	dsp_write(Y_sinusMode, 0x89a);
 }
 
+// consumes 1.7% of CPU time on 220MHz SA1100 when processing 176400 bytes/second
+void audio_mix( signed short* pDstSample, signed short* pSrcSample )
+{
+
+__asm__ (
+		"stmfd	r13!, {r0-r12,r14}\n\t"
+		
+		// initialize registers
+		"ldr	r1, %0\n\t"				// r1 = audio_overlay_bg_volume
+		"ldr	r4, %1\n\t"				// r4 = pDstSample
+		"ldr	r5, %2\n\t"				// r5 = pSrcSample
+		"mov	r6, #2304\n\t"			// r6 = loop counter
+		"sub	r6, r6, #1\n\t"			// r6 -= 1
+		"mov	r7, #65280\n\t"			// r7 = ...
+		"add	r7, r7, #255\n\t"		// r7 = 0xFFFF
+		"mov	r8, #32512\n\t"			// r8 = ...
+		"add	r8, r8, #255\n\t"		// r8 = 32767
+
+		".MixLoopStart:\n\t"
+
+		// scale destination sample down by fixed-point multiplier
+		"ldrsh	r0, [r4]\n\t"			// r0 = *pDstSample
+		"mov	r0, r0, lsl#16\n\t"		// r0 <<= 16
+		"smull  r2, r3, r1, r0\n\t"		// r2 = lo(r1 * r0), r3 = hi(r1 * r0)
+		"adc	r2, r2, #32768\n\t"		// r2 += 32768 + carry
+		"mov	r2, r2, lsr#16\n\t"		// r2 >>= 16
+		"and	r3, r3, r7\n\t"			// r3 &= r7
+		"orr	r0, r2, r3, lsl#16\n\t"	// r0 = r2 | (r3 << 16)
+		
+		// add source sample to scaled destination sample
+		"ldrsh	r2, [r5]\n\t"			// r2 = *pSrcSample
+		"add	r2, r2, r0, asr#16\n\t"	// r2 += (r0 >> 16)
+	
+		// clamp destination sample up
+		"cmp	r2, r8\n\t"				// if( r2 > 32767 )
+		"movgt	r2, r8\n\t"				//     r2 = 32767
+
+		// clamp destination sample down
+		"cmn	r2, r8\n\t"				// if( r2 < -32768 )
+		"mvnlt	r2, r8\n\t"				//     r2 = -32768
+
+		// put processed sample back to memory
+		"strh	r2, [r4]\n\t"			// *pDstSample = r2
+
+		"add	r4, r4, #2\n\t"			// r4 += 2 (get next dst sample pointer)
+		"add	r5, r5, #2\n\t"			// r5 += 2 (get next src sample pointer)
+
+		// loop back
+		"subs	r6, r6, #1\n\t"			// r6 -= 1
+		"bpl	.MixLoopStart\n\t"
+
+		"ldmfd	r13!, {r0-r12,r14}\n\t"
+		: // no outputs
+		: "m" (audio_overlay_bg_volume), "m" (pDstSample), "m" (pSrcSample) );
+}
+
+int audio_overlay_in_use()
+{
+	return audio_overlay.used > 0;
+}
+
+// copies audio data to dma; if audio buffer and overlay buffer both has
+// data, it mixes them together and outputs to dma; if only one has data, it
+// outputs that into dma and if both are empty, it outputs zero sample to dma
+// parameter: dma_register == 0 == DBSA0/DBTA0, dma_register == 1 == DBSB0/DBTB0
+void audio_data_to_dma( int dma_register )
+{
+	static int iSwitchToPCM = 0;
+	static int iFadeVolumeUp = 0;
+	static int iStoredVolume = 0;
+	static int iSam = 0;  // Did we change SAM?
+	int iPreviousOverlayUsed = audio_overlay.used;
+	audio_dev *dev=&audio[0];
+
+	if ( audio_overlay.used > 0 && !iSam ) {
+		iSam = 1;
+		empeg_mixer_setsam(0); // Disable SAM whilst overlay is active.
+	}
+	else if ( audio_overlay.used == 0 && iSam ) {
+		iSam = 0;
+		empeg_mixer_setsam(sam); // Restore player's SAM setting.
+	}
+
+
+
+	if( empeg_mixer_get_input() != 1 ) // INPUT_PCM
+	{
+		if( audio_overlay.used > 0 && !iSwitchToPCM )
+		{
+#if AUDIO_OVERLAY_DEBUG
+            printk("audio_overlay.used > 0 && !iSwitchToPCM\n");
+#endif
+			if( !iFadeVolumeUp )
+				iStoredVolume = empeg_mixer_getvolume();
+			
+            // start fading volume towards 0
+			iSwitchToPCM = 1;
+			iFadeVolumeUp = 0;
+		}
+		// output regular PCM data (should be silent as non-PCM output is active)
+		if( dev->used > 0 )
+		{
+			if( dma_register )
+		    	DBSB0=(unsigned char*)virt_to_phys(dev->buffers[dev->tail].data);
+		    else
+		    	DBSA0=(unsigned char*)virt_to_phys(dev->buffers[dev->tail].data);
+			if (++dev->tail==AUDIO_NOOF_BUFFERS) dev->tail=0;
+			dev->used--; dev->free++;
+		}
+		else
+		{
+			if( dma_register )
+				DBSB0=(unsigned char*)_ZeroMem;
+			else
+				DBSA0=(unsigned char*)_ZeroMem;
+		}
+	}
+	else
+	{
+
+		// dsp mode is PCM
+		if( dev->used == 0 )
+		{
+			if( audio_overlay.used == 0 )
+			{
+				if( dma_register )
+					DBSB0=(unsigned char*)_ZeroMem;
+				else
+					DBSA0=(unsigned char*)_ZeroMem;
+			}
+			else
+			{
+				//audio_overlay_bg_volume = AUDIO_OVERLAY_BG_VOLUME_FADED;
+                audio_overlay_bg_volume = hijack_overlay_bg_min;
+				if( dma_register )
+		    		DBSB0=(unsigned char*)virt_to_phys(audio_overlay.buffers[audio_overlay.tail].data);
+		    	else
+		    		DBSA0=(unsigned char*)virt_to_phys(audio_overlay.buffers[audio_overlay.tail].data);
+				if( ++audio_overlay.tail == AUDIO_OVERLAY_BUFFERS ) audio_overlay.tail = 0;
+				audio_overlay.used--; audio_overlay.free++;
+			}
+		}
+		else
+		{
+			if( audio_overlay.used > 0 )
+			{
+
+				//if( audio_overlay_bg_volume > AUDIO_OVERLAY_BG_VOLUME_FADED )
+                if( audio_overlay_bg_volume > hijack_overlay_bg_min )
+				{   
+                  
+                    // fading down...
+					audio_overlay_bg_volume -= hijack_overlay_bg_fadestep;
+					if( audio_overlay_bg_volume < hijack_overlay_bg_min )
+						audio_overlay_bg_volume = hijack_overlay_bg_min;
+					audio_mix( (signed short*)dev->buffers[ dev->tail ].data, &audio_zero_sample[0] );
+				}
+				else
+				{
+					audio_mix( (signed short*)dev->buffers[ dev->tail ].data, 
+							   (signed short*)audio_overlay.buffers[ audio_overlay.tail ].data );
+
+					if( ++audio_overlay.tail == AUDIO_OVERLAY_BUFFERS ) audio_overlay.tail = 0;
+					audio_overlay.used--; audio_overlay.free++;
+				}
+			}
+			else
+			{
+				if( audio_overlay_bg_volume < hijack_overlay_bg_max )
+				{
+                    // fading up...
+                    audio_overlay_bg_volume += hijack_overlay_bg_fadestep;
+					if( audio_overlay_bg_volume > hijack_overlay_bg_max )
+						audio_overlay_bg_volume = hijack_overlay_bg_max;
+					audio_mix( (signed short*)dev->buffers[ dev->tail ].data, &audio_zero_sample[0] );
+
+				}
+			}
+			
+            if( dma_register )
+		    	DBSB0=(unsigned char*)virt_to_phys(dev->buffers[dev->tail].data);
+		    else
+		    	DBSA0=(unsigned char*)virt_to_phys(dev->buffers[dev->tail].data);
+			if (++dev->tail==AUDIO_NOOF_BUFFERS) dev->tail=0;
+			dev->used--; dev->free++;
+		}
+        
+		// if all overlay buffers are played and user DSP mode is different, do a change		
+		if( audio_overlay.used == 0 && iPreviousOverlayUsed > 0 &&
+		    empeg_mixer_get_input() != empeg_mixer_get_user_input() )
+		{
+			if( !iFadeVolumeUp )
+			{
+#if AUDIO_OVERLAY_DEBUG
+                printk("!iFadeVolumeUp\n");
+#endif
+				iStoredVolume = empeg_mixer_getvolume();
+				empeg_mixer_setvolume( 0 );
+				empeg_mixer_select_input( empeg_mixer_get_user_input() );
+				// fade volume back to stored volume
+				iFadeVolumeUp = 1;
+			}
+		}
+	}
+
+	if( dma_register )
+		DBTB0=AUDIO_BUFFER_SIZE;
+	else
+		DBTA0=AUDIO_BUFFER_SIZE;
+
+	if( iSwitchToPCM )
+	{
+        // fade volume towards 0...
+		int iVol = empeg_mixer_getvolume();
+		iVol -= 7; // takes roughly 0.3sec to fade from 100 to 0
+		if( iVol < 0 )
+		{
+#if AUDIO_OVERLAY_DEBUG
+            printk("iVol < 0\n");
+#endif
+			empeg_mixer_select_input( 1 ); // INPUT_PCM
+			empeg_mixer_setvolume( iStoredVolume + 30 );
+			iSwitchToPCM = 0;
+		}
+		else
+			empeg_mixer_setvolume( iVol );
+	}
+	else if( iFadeVolumeUp )
+	{
+		// fade volume towards stored volume...
+        int iVol = empeg_mixer_getvolume();
+#if AUDIO_OVERLAY_DEBUG
+        printk("!iFadeVolumeUp\n");
+#endif
+		iVol += 7; // takes roughly 0.3sec to fade from 0 to 100
+		if( iVol > iStoredVolume )
+		{
+#if AUDIO_OVERLAY_DEBUG
+            printk("iVol > iStoredVolume\n");
+#endif
+			iVol = iStoredVolume;
+			iFadeVolumeUp = 0;
+		}
+		empeg_mixer_setvolume( iVol );
+	}
+}
+
 /*                      
  * Interrupt processing 
  */
@@ -1122,44 +1467,18 @@
         if (dofirst== 0) {
 		ClrDCSR0 = DCSR_DONEB;
 		
-		/* Any data to get? */
-		if (dev->used == 0) {
-			DBSA0 = (unsigned char *) _ZeroMem;
-			DBTA0 = AUDIO_BUFFER_SIZE;
-
-
-			/* If we've underrun, take note */
-			if (dev->good_data) {
-				dev->good_data = 0;
-				dev->stats.user_underruns++;
-			}
-		}
-		else {
-		        DBSA0 =	(unsigned char *)
-				virt_to_phys(dev->buffers[dev->tail].data);
-			DBTA0 = AUDIO_BUFFER_SIZE;
-			if (++dev->tail == AUDIO_NOOF_BUFFERS) dev->tail = 0;
-			dev->used--;
-			dev->free++;
+		/* If we've underrun, take note */
+		if( dev->used == 0 && dev->good_data )
+		{
+			dev->good_data = 0;
+			dev->stats.user_underruns++;
 		}
+		audio_data_to_dma( 0 );
 		
 		if (!(status & DCSR_STRTB)) {
 			/* Filling both buffers: possible IRQ underrun */
 			dev->stats.irq_underruns++;
-
-			if (dev->used == 0) {
-				DBSB0 = (unsigned char *) _ZeroMem;
-				DBTB0 = AUDIO_BUFFER_SIZE;
-			}
-			else {
-				DBSB0 = (unsigned char *) virt_to_phys(
-					dev->buffers[dev->tail].data);
-				DBTB0 = AUDIO_BUFFER_SIZE;
-				if (++dev->tail == AUDIO_NOOF_BUFFERS)
-					dev->tail = 0;
-				dev->used--;
-				dev->free++;
-			}
+			audio_data_to_dma( 1 );
 			
 			/* Start both channels */
 			SetDCSR0 =
@@ -1172,44 +1491,18 @@
 	else {
 		ClrDCSR0 = DCSR_DONEA;
 
-		/* Any data to get? */
-		if (dev->used == 0) {
-			DBSB0 = (unsigned char *) _ZeroMem;
-			DBTB0 = AUDIO_BUFFER_SIZE;
-
-			/* If we've underrun, take note */
-			if (dev->good_data) {
-				dev->good_data = 0;
-				dev->stats.user_underruns++;
-			}
-		}
-		else {
-			DBSB0 = (unsigned char *)
-				virt_to_phys(dev->buffers[dev->tail].data);
-			DBTB0 = AUDIO_BUFFER_SIZE;
-			if (++dev->tail == AUDIO_NOOF_BUFFERS)
-				dev->tail=0;
-			dev->used--;
-			dev->free++;
+		/* If we've underrun, take note */
+		if( dev->used == 0 && dev->good_data )
+		{
+			dev->good_data = 0;
+			dev->stats.user_underruns++;
 		}
+		audio_data_to_dma( 1 );
 		
 		if (!(status & DCSR_STRTA)) {
 			/* Filling both buffers: possible IRQ underrun */
 			dev->stats.irq_underruns++;
-
-			if (dev->used == 0) {
-				DBSA0 = (unsigned char *) _ZeroMem;
-				DBTA0 = AUDIO_BUFFER_SIZE;
-			}
-			else {
-				DBSA0 = (unsigned char*) virt_to_phys(
-					dev->buffers[dev->tail].data);
-				DBTA0 = AUDIO_BUFFER_SIZE;
-				if (++dev->tail == AUDIO_NOOF_BUFFERS)
-					dev->tail=0;
-				dev->used--;
-				dev->free++;
-			}
+			audio_data_to_dma( 0 );
 
 			/* Start both channels */
 			SetDCSR0 =
diff -u --recursive empeg.v2.00beta13.hijack/arch/arm/special/empeg_audio3.h empeg.v2.00beta13.audiooverlay/arch/arm/special/empeg_audio3.h
--- empeg.v2.00beta13.hijack/arch/arm/special/empeg_audio3.h	Thu Nov 21 12:46:50 2002
+++ empeg.v2.00beta13.audiooverlay/arch/arm/special/empeg_audio3.h	Thu Nov 28 00:16:53 2002
@@ -9,4 +9,6 @@
 int empeg_audio_open(struct inode *inode, struct file *file);
 void empeg_audio_beep_setup(int rate);
 
+int audio_overlay_in_use(void);
+
 #endif
diff -u --recursive empeg.v2.00beta13.hijack/arch/arm/special/empeg_mixer.c empeg.v2.00beta13.audiooverlay/arch/arm/special/empeg_mixer.c
--- empeg.v2.00beta13.hijack/arch/arm/special/empeg_mixer.c	Thu Nov 21 12:51:42 2002
+++ empeg.v2.00beta13.audiooverlay/arch/arm/special/empeg_mixer.c	Fri Nov 29 22:39:03 2002
@@ -63,6 +63,8 @@
 typedef struct
 {
 	int input;
+	int inputInternal;
+	int userInputInternal;
 	unsigned int flags;
 	int volume;
 	int loudness;
@@ -114,6 +116,7 @@
 static struct empeg_eq_section_t eq_current[20];
 static int eq_section_order[20];
 static int mixer_compression = 0;
+	int sam;
 static unsigned int eq_last[40];
 static unsigned int eq_reg_last = 0;
 static unsigned int radio_sensitivity;
@@ -137,9 +140,9 @@
 static int empeg_mixer_ioctl(struct inode *inode, struct file *file,
 			     uint command, ulong arg);
 
-       void empeg_mixer_select_input(int input);
+//static void empeg_mixer_select_input(int input);
 static void empeg_mixer_setloudness(mixer_dev *dev, int level);
-static int empeg_mixer_setvolume(mixer_dev *dev, int vol);
+//static int empeg_mixer_setvolume(mixer_dev *dev, int vol);
 static int empeg_mixer_getdb(mixer_dev *dev);
 static void empeg_mixer_inflict_flags(mixer_dev *dev);
 static void empeg_mixer_setbalance(mixer_dev *dev, int balance);
@@ -166,6 +169,7 @@
 };
 
 
+
 int __init empeg_mixer_init(void)
 {
 	mixer_dev *dev = &mixer_global;
@@ -204,7 +208,7 @@
 	empeg_mixer_mute(1);
 
 	/* Set volume */
-	empeg_mixer_setvolume(dev,VOLUME_ZERO_DB);
+	empeg_mixer_setvolume(VOLUME_ZERO_DB);
 
 	/* try doing this last thing */
 	empeg_mixer_select_input(INPUT_PCM);
@@ -409,7 +413,7 @@
 		if (vol_out < 0 || vol_out > 100)
 			return -EINVAL;
 		vol_out = hijack_volume_boost_w(vol_out);       // calculate boost volume
-		empeg_mixer_setvolume(dev, vol_out);
+		empeg_mixer_setvolume(vol_out);
 		vol_out = dev->volume;
 		vol_out = hijack_volume_boost_r(vol_out);	// 'de'calculate hijack volume boost. player sees what it asked for.
 		vol_out = (vol_out & 0xFF) + ((vol_out << 8));
@@ -470,24 +474,27 @@
 #endif	
 		if (source & SOUND_MASK_PCM)
 		{
-			empeg_mixer_select_input(INPUT_PCM);
+			dev->userInputInternal = INPUT_PCM;
 			dev->input = SOUND_MASK_PCM;
 		}
 		else if (source & SOUND_MASK_RADIO)
 		{
-			empeg_mixer_select_input(INPUT_RADIO_FM);
+			dev->userInputInternal = INPUT_RADIO_FM;
 			dev->input = SOUND_MASK_RADIO;
 		}
 		else if (source & SOUND_MASK_LINE1)
 		{
-			empeg_mixer_select_input(INPUT_RADIO_AM);
+			dev->userInputInternal = INPUT_RADIO_AM;
 			dev->input = SOUND_MASK_LINE1;
 		}
 		else if (source & SOUND_MASK_LINE)
 		{
-		        empeg_mixer_select_input(INPUT_AUX);
+			dev->userInputInternal = INPUT_AUX;
 			dev->input = SOUND_MASK_LINE;
 		}
+		if( dev->userInputInternal != dev->inputInternal && !audio_overlay_in_use() )
+			empeg_mixer_select_input( dev->userInputInternal );
+		
 		put_user_ret(dev->input, (int *)arg, -EFAULT);
 		return 0;
 	}
@@ -614,20 +621,36 @@
 		mixer_compression = onoff;
 		return 0;
 	}
+      
+      
+      case EMPEG_MIXER_GET_SAM:
+      {
+
+        int sam;
+    #if MIXER_DEBUG
+            printk(MIXER_NAME
+                   ": mixer_ioctl EMPEG_MIXER_GET_SAM %d\n",
+                   arg);
+    #endif
+          dsp_read_xram(Y_switch, &sam);
+          copy_to_user_ret((void *) arg, (const void *) &sam , sizeof(int), -EFAULT);
+          return 0;
+      }
+
 	case EMPEG_MIXER_SET_SAM:
 	{
-		int sam;
+        /* int sam;     Now global */
 
-		copy_from_user_ret((void *) &sam, (const void *) arg,
-				   sizeof(int), -EFAULT);
+        copy_from_user_ret((void *) &sam, (const void *) arg,
+                   sizeof(int), -EFAULT);
+
+            if(sam) dsp_write(Y_switch, 0);
+        else dsp_write(Y_switch, 0x5d4);	// 4.6ms
 
-       		if(sam) dsp_write(Y_switch, 0);
-		else dsp_write(Y_switch, 0x5d4);	// 4.6ms
-		
 #if MIXER_DEBUG
-		printk(MIXER_NAME
-		       ": mixer_ioctl EMPEG_MIXER_SET_SAM %d\n",
-		       sam);
+      printk(MIXER_NAME
+             ": mixer_ioctl EMPEG_MIXER_SET_SAM %d\n",
+             sam);
 #endif
 
 		return 0;
@@ -868,6 +891,29 @@
 #endif
 }
 
+
+void empeg_mixer_setsam(int on)
+{	
+	if(on) dsp_write(Y_switch, 0);
+	else dsp_write(Y_switch, 0x5d4);	// 4.6ms
+
+#if MIXER_DEBUG
+printk(MIXER_NAME
+       ": empeg_mixer_setsam %d\n",
+       sam);
+#endif
+}
+
+int empeg_mixer_get_input()
+{
+	return mixer_global.inputInternal;
+}
+
+int empeg_mixer_get_user_input()
+{
+	return mixer_global.userInputInternal;
+}
+
 void empeg_mixer_select_input(int input)
 {
 	static dsp_setup fm_setup[]=
@@ -916,6 +962,7 @@
 	  { 0,0 } };
 
 	mixer_dev *dev = &mixer_global;
+	dev->inputInternal = input;
 
 	eq_reg_last &= 0x1000; /* Preserve EQ num bands bit */
 	eq_reg_last |= (radio_sensitivity << 1) | (radio_sensitivity << 4);
@@ -1119,7 +1166,8 @@
 		/* Wait for a while so that the I2S being clocked into the
 		   DSP by DMA runs the initialisation code: the DSP is cycle-
 		   locked to the incoming bitstream */
-		{ int a=jiffies+(HZ/20); while(jiffies<a); }
+//		{ int a=jiffies+(HZ/20); while(jiffies<a); }
+		{ int a=0; for( a = 0; a < 1000; ++a ) udelay( 50 ); } // jiffies can not be used inside interrupt
 		
 		/* Select mode */
 		dsp_write(X_modpntr,0x0200);
@@ -1200,15 +1248,17 @@
 	// Calculate the desired boost for the new input.
 	int vol;
 	vol = hijack_volume_boost_w(hijack_current_vol_request);
-	(void) empeg_mixer_setvolume(dev, vol);
+	(void) empeg_mixer_setvolume(vol);
 }
 
-static int empeg_mixer_setvolume(mixer_dev *dev, int vol)
+
+/*static*/ 
+int empeg_mixer_setvolume(/*mixer_dev *dev,*/ int vol)
 {
 	hijack_current_mixer_volume = vol;
 	dsp_write(Y_VAT, volume_table[vol].vat);
 	dsp_write(Y_VGA, volume_table[vol].vga);
-	dev->volume = vol;
+	mixer_global.volume = vol;	
 
 #if MIXER_DEBUG
 	printk(MIXER_NAME ": volume set %d VAT:%03x VGA:%03x\n",vol,
@@ -1258,6 +1308,29 @@
 		(void)empeg_mixer_eq_apply();
 	}
 }
+
+int empeg_mixer_getvolume(void)
+{
+	return mixer_global.volume;
+}
+
+int empeg_mixer_getsam(void)
+{
+  
+  int sam;
+  
+  dsp_read_xram(Y_switch, &sam);
+  if (sam)
+  {
+    return 0;
+  } else {
+    return 1;
+  }
+  
+
+}
+
+
 
 static int empeg_mixer_getdb(mixer_dev *dev)
 {
diff -u --recursive empeg.v2.00beta13.hijack/arch/arm/special/empeg_mixer.h empeg.v2.00beta13.audiooverlay/arch/arm/special/empeg_mixer.h
--- empeg.v2.00beta13.hijack/arch/arm/special/empeg_mixer.h	Thu Nov 21 12:51:42 2002
+++ empeg.v2.00beta13.audiooverlay/arch/arm/special/empeg_mixer.h	Thu Nov 28 00:16:18 2002
@@ -23,4 +23,16 @@
 #define INPUT_AUX			2
 #define INPUT_RADIO_AM			3
 
+void empeg_mixer_select_input(int input);
+int empeg_mixer_get_input(void);
+int empeg_mixer_get_user_input(void);
+
+int empeg_mixer_setvolume(int vol);
+int empeg_mixer_getvolume(void);
+
+int empeg_mixer_getsam(void);
+void empeg_mixer_setsam(int sam);
+
+void empeg_mixer_setsam (int on);
+
 #endif
diff -u --recursive empeg.v2.00beta13.hijack/arch/arm/special/hijack.c empeg.v2.00beta13.audiooverlay/arch/arm/special/hijack.c
--- empeg.v2.00beta13.hijack/arch/arm/special/hijack.c	Thu Nov 21 12:51:42 2002
+++ empeg.v2.00beta13.audiooverlay/arch/arm/special/hijack.c	Wed Nov 27 23:28:55 2002
@@ -554,6 +554,11 @@
 // Tony bonus. Give the option of removing default bass boost on FM  -- genixia
        int hijack_disable_bassboost_FM;
 
+// audio overlay patch settings
+       int hijack_overlay_bg_min = 0x00004000;
+       int hijack_overlay_bg_max = 0x00010000;
+       int hijack_overlay_bg_fadestep = 0x00000AAA;
+
 typedef struct hijack_option_s {
 	char	*name;
 	void	*target;
@@ -623,6 +628,10 @@
 {"volume_boost_AUX",		&hijack_volboost[2],		0,			1,	-100,	100},
 {"volume_boost_AM",		&hijack_volboost[3],		0,			1,	-100,	100},
 {"disable_bassboost_FM",	&hijack_disable_bassboost_FM,	0,			1,	0,	1,},
+{"overlay_bg_min",	&hijack_overlay_bg_min,	0x00004000, 1, 0, 0x00010000,},
+{"overlay_bg_max",	&hijack_overlay_bg_max,	0x00010000, 1, 0, 0x00010000,},
+{"overlay_bg_fadestep",	&hijack_overlay_bg_max,	0x00000AAA, 1, 0, 0x00010000,},
+
 {NULL,NULL,0,0,0,0} // end-of-list
 };
 
diff -u --recursive empeg.v2.00beta13.hijack/include/linux/empeg.h empeg.v2.00beta13.audiooverlay/include/linux/empeg.h
--- empeg.v2.00beta13.hijack/include/linux/empeg.h	Thu Nov 21 12:51:51 2002
+++ empeg.v2.00beta13.audiooverlay/include/linux/empeg.h	Wed Nov 27 23:28:55 2002
@@ -98,6 +98,7 @@
 #define EMPEG_MIXER_READ_SIGNAL_STRENGTH_FAST _IOR(EMPEG_MIXER_MAGIC, 24, int)
 #define EMPEG_MIXER_READ_FM_DEEMPHASIS	_IOR(EMPEG_MIXER_MAGIC, 25, int)
 #define EMPEG_MIXER_WRITE_FM_DEEMPHASIS	_IOW(EMPEG_MIXER_MAGIC, 25, int)
+#define EMPEG_MIXER_GET_SAM		_IOR(EMPEG_MIXER_MAGIC, 26, int)
 
 /* Retrieve volume level corresponding to 0dB */
 #define EMPEG_MIXER_READ_ZERO_LEVEL _IOR(EMPEG_MIXER_MAGIC, 3, int)
